#
#  Python script
#  Generates variables to be used in the template file
#  from an array of transactions dictionaries
#  
#  Write your own variablesFromTransactions_(self, transactions):
#  to customize the available variables
#

from Cocoa import *

ReportGenerator = objc.lookUpClass("NSObject")
class CategoryDetailsReportGenerator(ReportGenerator):
	@classmethod
	def defaultGenerator(self):
		generator = CategoryDetailsReportGenerator.new()
		return generator
	def variablesFromTransactions_(self, transactions):
		return {'transactions':transactions, 'categories':self.categoriesFromTransactions_(transactions)}
	def categoriesFromTransactions_(self, transactions):
		categories = transactions.valueForKeyPath_("@distinctUnionOfObjects.category")
		categoriesDictionaries = NSMutableArray.alloc().initWithCapacity_(categories.count())
		for category in categories:
			transactionsForThisCategoryPredicate = NSPredicate.predicateWithFormat_("category == %@", category)
			transactionsForThisCategory = transactions.filteredArrayUsingPredicate_(transactionsForThisCategoryPredicate)
			
			amountForThisCategory = transactionsForThisCategory.valueForKeyPath_("@sum.amount")
			
			dateSortDescriptor = NSSortDescriptor.alloc().initWithKey_ascending_("date", True)
			transactionsSortedByDate = transactionsForThisCategory.sortedArrayUsingDescriptors_([dateSortDescriptor])
			
			categoriesDictionaries.addObject_({'amount':amountForThisCategory,
			'transactionsSortedByDate':transactionsSortedByDate,
			'name':category.name()})
		nameSortDescriptor = NSSortDescriptor.alloc().initWithKey_ascending_("name", True)
		categoriesDictionaries.sortUsingDescriptors_([nameSortDescriptor])
		return categoriesDictionaries