#
#  Python script
#  Generates variables to be used in the template file
#  from an array of transactions dictionaries
#  
#  Write your own variablesFromTransactions_(self, transactions):
#  to customize the available variables
#

from Cocoa import *

ReportGenerator = objc.lookUpClass("NSObject")
class PieChartsReportGenerator(ReportGenerator):
	@classmethod
	def defaultGenerator(self):
		generator = PieChartsReportGenerator.new()
		return generator
	def variablesFromTransactions_(self, transactions):
		categoriesDictionaries = self.categoriesFromTransactions_(transactions)
		return {'transactions':transactions.sortedArrayUsingDescriptors_([NSSortDescriptor.alloc().initWithKey_ascending_("date", True)]), 'categories':categoriesDictionaries, 'totalIncome':self.totalIncome_(categoriesDictionaries), 'totalExpense':self.totalExpenditures_(categoriesDictionaries), 'totalAmount':transactions.valueForKeyPath_("@sum.amount")}
	def categoriesFromTransactions_(self, transactions):
		categories = transactions.valueForKeyPath_("@distinctUnionOfObjects.category")
		categoriesDictionaries = NSMutableArray.alloc().initWithCapacity_(categories.count())
		for category in categories:
			transactionsForThisCategoryPredicate = NSPredicate.predicateWithFormat_("category == %@", category)
			transactionsForThisCategory = transactions.filteredArrayUsingPredicate_(transactionsForThisCategoryPredicate)
			
			amountForThisCategory = transactionsForThisCategory.valueForKeyPath_("@sum.amount")
			
			dateSortDescriptor = NSSortDescriptor.alloc().initWithKey_ascending_("date", True)
			transactionsSortedByDate = transactionsForThisCategory.sortedArrayUsingDescriptors_([dateSortDescriptor])
			
			categoriesDictionaries.addObject_({'amount':amountForThisCategory,
			'color':category.color(),
			'name':category.name()})
		categoriesDictionaries.sortUsingDescriptors_([NSSortDescriptor.alloc().initWithKey_ascending_("name", True)])
		return categoriesDictionaries
	def totalIncome_(self, categoriesDictionaries):
		incomesPredicate = NSPredicate.predicateWithFormat_("amount >= 0")
		incomeCategories = categoriesDictionaries.filteredArrayUsingPredicate_(incomesPredicate)
		return incomeCategories.valueForKeyPath_("@sum.amount")
	def totalExpenditures_(self, categoriesDictionaries):
		expendituresPredicate = NSPredicate.predicateWithFormat_("amount <= 0")
		expendituresCategories = categoriesDictionaries.filteredArrayUsingPredicate_(expendituresPredicate)
		return expendituresCategories.valueForKeyPath_("@sum.amount").decimalNumberByMultiplyingBy_(NSDecimalNumber.alloc().initWithString_("-1"))
	